package be.iminds.ilabt.jfed.highlevel.tasks;

import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.highlevel.util.JavaFXLogger;
import be.iminds.ilabt.jfed.log.ResultListener;
import be.iminds.ilabt.jfed.lowlevel.api.AbstractFederationApi;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.UserAndSliceApiWrapper;
import be.iminds.ilabt.jfed.lowlevel.connection.JFedException;
import be.iminds.ilabt.jfed.util.common.GeniUrn;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.time.Instant;
import java.util.*;

/**
 * User: twalcari
 * Date: 6/17/2015
 * Time: 15:21
 */
public final class RegisterSliceRspecTask extends Task {
    private static final org.slf4j.Logger LOG = LoggerFactory.getLogger(RegisterSliceRspecTask.class);

    @Nonnull
    private final AbstractFederationApi.SliceRspecType type;
    @Nonnull
    private final Slice slice;
    @Nullable
    private final Server auth;
    @Nonnull
    private final String rspec;
    @Nullable
    private final Task extraTaskToDependOn;
    @Nonnull
    private final Instant sliceProvisionDate;
    @Nonnull
    private final ResultListener[] extraResultListeners;

    private final UserAndSliceApiWrapper userAndSliceApiWrapper;
    private final JavaFXLogger logger;
    private final HighLevelTaskFactory highLevelTaskFactory;

    public RegisterSliceRspecTask(@Nonnull AbstractFederationApi.SliceRspecType type,
                                  @Nonnull Slice slice,
                                  @Nullable Server auth,
                                  @Nonnull String rspec,
                                  @Nonnull Instant sliceProvisionDate,
                                  @Nullable Task extraTaskToDependOn,
                                  @Nonnull UserAndSliceApiWrapper userAndSliceApiWrapper,
                                  @Nonnull JavaFXLogger logger,
                                  @Nonnull HighLevelTaskFactory highLevelTaskFactory,
                                  @Nonnull ResultListener... extraResultListeners) {
        super("Registering slice RSpec for " + (auth == null ? "(NO AUTH)" : auth.getDefaultComponentManagerUrn()));

        assert (auth == null) == (type != AbstractFederationApi.SliceRspecType.MANIFEST) : "auth is required for MANIFEST, but not allowed for REQUEST and COMBINED MANIFEST";

        this.type = type;
        this.slice = slice;
        this.rspec = rspec;
        this.sliceProvisionDate = sliceProvisionDate;
        this.auth = auth;
        this.extraTaskToDependOn = extraTaskToDependOn;

        this.userAndSliceApiWrapper = userAndSliceApiWrapper;
        this.logger = logger;
        this.highLevelTaskFactory = highLevelTaskFactory;
        this.extraResultListeners = extraResultListeners;
    }

    @Override
    protected void doTask(@Nonnull TaskExecution taskExecution) throws JFedException, InterruptedException {
        ResultListener[] resultListeners = Arrays.copyOf(extraResultListeners, extraResultListeners.length + 1);
        resultListeners[extraResultListeners.length] = taskExecution;
        JavaFXLogger logger = JavaFXLogger.wrap(this.logger, resultListeners);

        userAndSliceApiWrapper.registerRspec(logger,
                slice.getCredentials(),
                type, slice.getUrn(),
                auth == null ? null : auth.getDefaultComponentManagerAsGeniUrn(),
                rspec,
                Date.from(sliceProvisionDate));
    }

    @Nonnull
    @Override
    public List<Task> initDependsOn() {
        List<Task> res = new ArrayList<>();
        if (extraTaskToDependOn != null)
            res.add(extraTaskToDependOn);
        res.add(highLevelTaskFactory.getSliceCredential(slice));
        return res;
    }
}
